/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            daemon.cc
 *
 *  Thu Jun  9 10:27:59 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "daemon.h"

#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>

Daemon::Daemon()
{}

Daemon::~Daemon()
{}

int Daemon::run(uid_t uid, gid_t gid)
{
  int f;
  int fd;

  chdir("/");
  umask(0);

  f = fork();
  switch(f) {
  case -1: // Fork error
    perror("Fork in daemon.cc");
    return 1;

  case 0:  // Forked child
    if(setgid(gid) != 0) {
      fprintf(stderr, "Failed to change to gid %d, quitting.\n", gid);
      return 1;
    }
    
    if(setuid(uid) != 0) {
      fprintf(stderr, "Failed to change to uid %d, quitting.\n", uid);
      return 1;
    }
    
    // Redirect stdin, stdout and stderr to /dev/null
    fd = open("/dev/null", O_NOCTTY | O_RDWR, 0666);

    dup2(0, fd);
    dup2(1, fd);
    dup2(2, fd);
    
    setsid();

    signal (SIGTERM, SIG_IGN);
    signal (SIGINT, SIG_IGN);
    signal (SIGHUP, SIG_IGN);

    return daemon_main();

  default: // Parent
    // exit(0);
    return 0;
  }
}
