/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            encoder.h
 *
 *  Thu Apr 14 19:29:55 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 * Originally from:
 * RTVideoRec Realtime video recoder and encoder for Linux
 *
 * Copyright (C) 2004  B. Stultiens
 */

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: encoder.h,v 1.13 2005/06/16 21:28:57 deva Exp $
 */

/*
 * $Log: encoder.h,v $
 * Revision 1.13  2005/06/16 21:28:57  deva
 * Rewrote thread object
 * Fixed bug in mov_encoder (pushed read_sem too many times, whihc lead to
 * growing server queue)
 *
 * Revision 1.12  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.11  2005/05/02 10:35:23  deva
 * Fixed wrongly showed snapshot thumbnails.
 *
 * Revision 1.10  2005/05/02 10:18:51  deva
 * Preserve network connection when a frozen frame exists, even though no recording is done.
 *
 * Revision 1.9  2005/05/02 09:50:22  deva
 * Rewrote freeze, shoot and record flags, from encoder to frame.
 *
 * Revision 1.8  2005/05/01 11:25:56  deva
 * Added code to read screenshot from frame queue, decode it to rgb and put it into a qimage.
 *
 * Revision 1.7  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include <config.h>
#ifdef USE_GUI

#ifndef __RTVIDEOREC_ENCODER_H
#define __RTVIDEOREC_ENCODER_H

#include "thread.h"

#include "info.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <avformat.h>

#include "miav.h"
#include "util.h"
#include "package.h"
#include "frame.h"
#include "queue.h"

// FIXME: One size fits all...
#define VIDEO_BUFFER_SIZE	(1024*1024)


/**
 * This class contains code for sending the video stream and the snapshots
 * over the network, to the MIaV server.
 */
class Encoder : public Thread {
public:
  Encoder(Info* ginfo,
          const char *gip,
          const int gport,
          sem_t	*gsem,
          Queue<Frame> *gqueue,
          pthread_mutex_t *gmutex,
          volatile int *grunning);
  ~Encoder();
  
  void setCpr(char *newcpr);

  void start();
  void stop(n_savestate save);

  void freeze();
  void shoot();

  void thread_main();

  //  AVFormatContext *fc;
  sem_t	*sem;
  Queue<Frame> *queue;
  pthread_mutex_t *mutex;
  volatile int *running;

private:
  Info *info;
  
  int port;
  char ip[32];
  char cpr[32];

  bool frozen;

  //  volatile int record;

  //  volatile int shoot_request;
  //  int shoot_value;
  //  volatile int freeze_request;
  //  int freeze_value;
  
  sem_t	record_sem;
  void encode();

  Socket *s;
  Network *n;
};

#endif

#endif /*USE_GUI*/
