/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            frame.h
 *
 *  Mon Nov 15 19:45:07 CET 2004
 *  Copyright  2004 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: frame.h,v 1.8 2005/06/16 21:28:57 deva Exp $
 */

/*
 * $Log: frame.h,v $
 * Revision 1.8  2005/06/16 21:28:57  deva
 * Rewrote thread object
 * Fixed bug in mov_encoder (pushed read_sem too many times, whihc lead to
 * growing server queue)
 *
 * Revision 1.7  2005/05/25 13:11:42  deva
 *
 * Made unfreeze close connection, when no recording is done.
 *
 * Revision 1.6  2005/05/22 15:49:22  deva
 * Added multithreaded encoding support.
 *
 * Revision 1.5  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.4  2005/05/02 09:50:22  deva
 * Rewrote freeze, shoot and record flags, from encoder to frame.
 *
 * Revision 1.3  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include <config.h>
#ifndef __FRAME_H__
#define __FRAME_H__

// Definition of vector
#include <vector>

// Definition of priority_queue
#include <queue>

class Frame {
public:
  Frame(unsigned char *d, int sz);
  ~Frame();
  
  unsigned char *data;
  int size;

  unsigned int number;

  bool shoot;
  int freeze; // 1 is freeze, -1 is unfreeze
  bool record;
};

#include <functional>

// Method for use, when comparing Frames in priority queue.
template <typename T>
struct frame_priority : std::binary_function<T, T, bool> {
  bool operator() (const T& a, const T& b) const {
    return ((Frame*)a)->number > ((Frame*)b)->number;
  }
};

// Additional helper types.
typedef std::vector< Frame* > FrameVector;
typedef std::queue< FrameVector* > FrameVectorQueue;
typedef std::priority_queue< Frame*, 
                             std::vector<Frame*>, 
                             frame_priority<Frame*> > FramePriorityQueue;

#endif/*__FRAME_H__*/
