/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            info_gui.cc
 *
 *  Tue May  3 09:34:59 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id$
 */

/*
 * $Log$
 */

#include <config.h>
#ifdef USE_GUI

#include "info_gui.h"

#include <stdio.h>
#include <stdarg.h>

#include <time.h>

bool InfoEventHandler::eventFilter( QObject *o, QEvent *e )
{
  if ( e->type() == TYPE_SHOW_MESSAGEBOX ) {
    //    fprintf(stderr, "Custom event!\n"); fflush(stderr);
    MessageBox *msgbox = ((ShowMessageEvent*)e)->messagebox();
    msgbox->exec();
    delete msgbox;
    return TRUE; // eat event
  } else {
    // standard event processing
    return FALSE;
  }
}

InfoGui::InfoGui(QApplication *a, QWidget *p, MiavConfig *c): Info()
{
  this->config = c;
  log_filename = *(this->config->readString("client_log_file"));

  qapp = a;
  parent = p;

  pthread_mutex_init (&mutex, NULL);
}

InfoGui::~InfoGui()
{
  pthread_mutex_destroy(&mutex);
}

void InfoGui::setParent(QWidget *p)
{
  parent = p;
}

void InfoGui::showmsg(char *msg, char *title, msg_icon icon)
{
  pthread_mutex_lock(&mutex);
  // Beginning of safezone

  fprintf(stderr, "%s: %s\n", title, msg); fflush(stderr);

  while( !parent ) {
    struct timespec ts;

    ts.tv_sec = 0;
    ts.tv_nsec = 200000000L;	// 200ms
    nanosleep(&ts, NULL);
  }

  MessageBox *msgbox = new MessageBox(parent, 
                                      title,
                                      msg, 
                                      TYPE_OK, 
                                      icon);

  ShowMessageEvent *event = new ShowMessageEvent( msgbox );
  
  qapp->postEvent(parent, event);

  // End of safezone
  pthread_mutex_unlock(&mutex);
}

void InfoGui::error(char *fmt, ...)
{
  char buf[1024];

	va_list argp;
	va_start(argp, fmt);
	vsprintf(buf, fmt, argp);
	va_end(argp);

  showmsg(buf, TXT_ERROR_TITLE, ICON_ERROR);

  log("Error: %s", buf);
}

void InfoGui::warn(char *fmt, ...)
{
  char buf[1024];

	va_list argp;
	va_start(argp, fmt);
	vsprintf(buf, fmt, argp);
	va_end(argp);

  showmsg(buf, TXT_WARNING_TITLE, ICON_WARNING);

  log("Warning: %s", buf);
}

void InfoGui::info(char *fmt, ...)
{
  char buf[1024];

	va_list argp;
	va_start(argp, fmt);
	vsprintf(buf, fmt, argp);
	va_end(argp);

  showmsg(buf, TXT_INFO_TITLE, ICON_INFO);

  log("Info: %s", buf);
}


#endif/*USE_GUI*/
