/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            mov_encoder.h
 *
 *  Sat Feb 19 14:13:19 CET 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 * Originally from:
 * RTVideoRec Realtime video recoder and encoder for Linux
 *
 * Copyright (C) 2004  B. Stultiens
 */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * $Id: mov_encoder.h,v 1.13 2005/06/16 21:54:22 deva Exp $
 */

/*
 * $Log: mov_encoder.h,v $
 * Revision 1.13  2005/06/16 21:54:22  deva
 * *** empty log message ***
 *
 * Revision 1.12  2005/06/16 21:28:57  deva
 * Rewrote thread object
 * Fixed bug in mov_encoder (pushed read_sem too many times, whihc lead to
 * growing server queue)
 *
 * Revision 1.11  2005/06/14 12:29:40  deva
 * Incorporated the use of the Info object everywhere... also using the log functionality.
 *
 * Revision 1.10  2005/05/22 15:49:22  deva
 * Added multithreaded encoding support.
 *
 * Revision 1.9  2005/05/19 14:10:22  deva
 * Multithreading rulez?
 *
 * Revision 1.8  2005/05/17 14:30:56  deva
 * Added code, preparing threaded encoding.
 *
 * Revision 1.7  2005/05/09 16:40:20  deva
 * Added optimize yuv conversion code
 *
 * Revision 1.6  2005/05/05 20:41:38  deva
 * Removed the last pieces of ffmpeg... replaced it with libfame...
 * Not quite working yet, but all the major code is in place!
 *
 * Revision 1.5  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.4  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#ifndef __RTVIDEOREC_ENCODER_H
#define __RTVIDEOREC_ENCODER_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// Use libfame
#include <fame.h>

// Use libdv
#include <libdv/dv.h>
#include <libdv/dv_types.h>

#include <vector>
using namespace std;

#include "frame.h"
#include "util.h"

#include "thread.h"
#include <pthread.h>

#include "info.h"

// size specifies the length of the buffer. 
#define FAME_BUFFER_SIZE	(1024*1024)	// FIXME: One size fits all...

class MovEncoder : public Thread {
public:
  MovEncoder(sem_t *r_sem,
             FrameVectorQueue *in, sem_t *in_sem, pthread_mutex_t *in_mutex,
             FramePriorityQueue *out, sem_t *out_sem, pthread_mutex_t *out_mutex,
             Info *info);
  ~MovEncoder();
  Frame* encode(Frame *frame);

  void thread_main();

  volatile bool running;

private:
  Info *info;

  // Input/Output queues
  FrameVectorQueue *inputqueue;
  FramePriorityQueue *outputqueue;

  //thread stuff
  sem_t *input_sem;
  sem_t *output_sem;

  sem_t *read_sem;
  
  pthread_mutex_t *input_mutex;
  pthread_mutex_t *output_mutex;

  Frame *encode_video(Frame *frame);
  void encode_audio(Frame *frame);

  // libFAME encoder
  //  unsigned char *fame_buffer;
  fame_parameters_t fame_par;
  fame_context_t *fame_context;
  fame_yuv_t yuv;
  unsigned char fame_buffer[FAME_BUFFER_SIZE];

  // libdv decoder
 	dv_decoder_t *dvdecoder;

  unsigned char picture[FAME_BUFFER_SIZE];
};

#endif

